@@ -1,6 +1,6 @@
 /*                                                   -*-c-*- */
 /*
- *   Copyright (C) 2003 Dale Mellor
+ *   Copyright (C) 2003, 2014 Dale Mellor
  * 
  *   This file is part of GNU mcron.
  *
@@ -66,7 +66,7 @@
 void
 react_to_terminal_signal (int sig)
 {
-  scm_eval_string (scm_take0str ("(delete-run-file)") );
+  scm_c_eval_string ("(delete-run-file)");
   exit (1);
 }
 
@@ -100,7 +100,7 @@ inner_main ()
 {
   scm_c_define_gsubr ("c-set-cron-signals", 0, 0, 0, set_cron_signals);
     
-  scm_eval_string (scm_take0str (
+  scm_c_eval_string (
 "(use-modules (mcron config))"
 "(if config-debug (begin (debug-enable 'debug)"
 "                        (debug-enable 'backtrace)))"
@@ -160,7 +160,7 @@ inner_main ()
 "\" command-name \"  (\" config-package-string \")\\n"
 "Written by Dale Mellor\\n"
 "\\n"
-"Copyright (C) 2003, 2006  Dale Mellor\\n"
+"Copyright (C) 2003, 2006, 2014  Dale Mellor\\n"
 "This is free software; see the source for copying conditions.  There is NO\\n"
 "warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\\n"
 "\"))"
@@ -173,14 +173,14 @@ inner_main ()
 "  ((mcron)"
 "\" [OPTIONS] [FILES]\\n"
 "Run an mcron process according to the specifications in the FILES (`-' for\\n"
-"standard input), or use all the files in ~/.cron with .guile or .vixie\\n"
-"extensions.\\n"
+"standard input), or use all the files in ~/.config/cron (or the \\n"
+"deprecated ~/.cron) with .guile or .vixie extensions.\\n"
 "\\n"
 "  -v, --version             Display version\\n"
 "  -h, --help                Display this help message\\n"
 "  -sN, --schedule[=]N       Display the next N jobs that will be run by mcron\\n"
-"  -d, --daemon              Immediately detach the program from the terminal and\\n"
-"                              run as a daemon process\\n"
+"  -d, --daemon              Immediately detach the program from the terminal\\n"
+"                              and run as a daemon process\\n"
 "  -i, --stdin=(guile|vixie) Format of data passed as standard input or\\n"
 "                              file arguments (default guile)\")"
 "  ((cron)"
@@ -236,50 +236,55 @@ inner_main ()
 "                                        (display in)))))"
 "(if (eq? command-type 'crontab)"
 "    (begin"
-"(define (hit-server user-name)"
-"  (catch #t (lambda ()"
-"              (let ((socket (socket AF_UNIX SOCK_STREAM 0)))"
-"                (connect socket AF_UNIX config-socket-file)"
-"                (display user-name socket)"
-"                (close socket)))"
-"         (lambda (key . args)"
-"           (display \"Warning: a cron daemon is not running.\\n\"))))"
-"(define (in-access-file? file name)"
-"  (catch #t (lambda ()"
-"              (with-input-from-file file (lambda ()"
-"                (let loop ((input (read-line)))"
-"                  (if (eof-object? input)"
-"                      #f"
-"                      (if (string=? input name)"
-"                          #t"
-"                          (loop (read-line))))))))"
-"         (lambda (key . args) '())))"
-"(define crontab-real-user (passwd:name (getpw (getuid))))"
-"(if (or (eq? (in-access-file? config-allow-file crontab-real-user) #f)"
-"        (eq? (in-access-file? config-deny-file crontab-real-user) #t))"
-"    (mcron-error 6 \"Access denied by system operator.\"))"
-"(if (> (+ (if (option-ref options 'edit #f) 1 0)"
-"          (if (option-ref options 'list #f) 1 0)"
-"          (if (option-ref options 'remove #f) 1 0))"
-"       1)"
-"    (mcron-error 7 \"Only one of options -e, -l or -r can be used.\"))"
-"(if (and (not (eqv? (getuid) 0))"
-"         (option-ref options 'user #f))"
-"    (mcron-error 8 \"Only root can use the -u option.\"))"
-"(define crontab-user (option-ref options 'user crontab-real-user))"
-"(define crontab-file (string-append config-spool-dir \"/\" crontab-user))"
-"(define (get-yes-no prompt . re-prompt)"
-"  (if (not (null? re-prompt))"
-"      (display \"Please answer y or n.\\n\"))"
-"  (display (string-append prompt \" \"))"
-"  (let ((r (read-line)))"
-"    (if (not (string-null? r))"
-"        (case (string-ref r 0)"
-"          ((#\\y #\\Y) #t)"
-"          ((#\\n #\\N) #f)"
-"          (else (get-yes-no prompt #t)))"
-"        (get-yes-no prompt #t))))"
-"(cond"
+"(let ((hit-server"
+"       (lambda (user-name)"
+"         (catch #t (lambda ()"
+"                     (let ((socket (socket AF_UNIX SOCK_STREAM 0)))"
+"                       (connect socket AF_UNIX config-socket-file)"
+"                       (display user-name socket)"
+"                       (close socket)))"
+"                (lambda (key . args)"
+"                  (display \"Warning: a cron daemon is not running.\\n\")))))"
+"      (in-access-file?"
+"       (lambda (file name)"
+"         (catch #t (lambda ()"
+"                     (with-input-from-file"
+"                         file"
+"                       (lambda ()"
+"                         (let loop ((input (read-line)))"
+"                           (if (eof-object? input)"
+"                               #f"
+"                               (if (string=? input name)"
+"                                   #t"
+"                                   (loop (read-line))))))))"
+"                (lambda (key . args) '()))))"
+"      (crontab-real-user (passwd:name (getpw (getuid)))))"
+"  (if (or (eq? (in-access-file? config-allow-file crontab-real-user) #f)"
+"          (eq? (in-access-file? config-deny-file crontab-real-user) #t))"
+"      (mcron-error 6 \"Access denied by system operator.\"))"
+"  (if (> (+ (if (option-ref options 'edit #f) 1 0)"
+"            (if (option-ref options 'list #f) 1 0)"
+"            (if (option-ref options 'remove #f) 1 0))"
+"         1)"
+"      (mcron-error 7 \"Only one of options -e, -l or -r can be used.\"))"
+"  (if (and (not (eqv? (getuid) 0))"
+"           (option-ref options 'user #f))"
+"      (mcron-error 8 \"Only root can use the -u option.\"))"
+"  (let ("
+"        (crontab-user (option-ref options 'user crontab-real-user))"
+"        (crontab-file (string-append config-spool-dir \"/\" crontab-user))"
+"        (get-yes-no (lambda (prompt . re-prompt)"
+"                      (if (not (null? re-prompt))"
+"                          (display \"Please answer y or n.\\n\"))"
+"                      (display (string-append prompt \" \"))"
+"                      (let ((r (read-line)))"
+"                        (if (not (string-null? r))"
+"                            (case (string-ref r 0)"
+"                              ((#\\y #\\Y) #t)"
+"                              ((#\\n #\\N) #f)"
+"                              (else (get-yes-no prompt #t)))"
+"                            (get-yes-no prompt #t))))))"
+"    (cond"
 " ((option-ref options 'list #f)"
 "  (catch #t (lambda ()"
 "              (with-input-from-file crontab-file (lambda ()"
@@ -332,6 +337,7 @@ inner_main ()
 "    (hit-server crontab-user)))"
 " (else"
 "  (mcron-error 15 \"usage error: file name must be specified for replace.\")))"
+")) "
 "      (quit)))"
 "(define (regular-file? file)"
 "  (catch 'system-error"
@@ -353,17 +359,27 @@ inner_main ()
 "        ((regexp-exec vixie-file-regexp file-path)"
 "         (read-vixie-file file-path))))"
 "(define (process-files-in-user-directory)"
-"  (catch #t (lambda ()"
-"              (let* ((dir-path (string-append (passwd:dir (getpw (getuid)))"
-"                                              \"/.cron\"))"
-"                     (directory (opendir dir-path)))"
-"                (do ((file-name (readdir directory) (readdir directory)))"
-"                    ((eof-object? file-name) (closedir directory))"
-"                  (process-user-file (string-append dir-path"
-"                                                    \"/\""
-"                                                    file-name)))))"
-"         (lambda (key . args)"
-"           (mcron-error 13 \"Cannot read files in your ~/.cron directory.\"))))"
+"  (let ((errors 0)"
+"        (home-directory (passwd:dir (getpw (getuid)))))"
+"    (map (lambda (config-directory)"
+"          (catch #t"
+"                 (lambda ()"
+"                   (let ((directory (opendir config-directory)))"
+"                     (do ((file-name (readdir directory) (readdir directory)))"
+"                         ((eof-object? file-name) (closedir directory))"
+"                       (process-user-file (string-append config-directory"
+"                                                         \"/\""
+"                                                         file-name)))))"
+"                 (lambda (key . args)"
+"                   (set! errors (1+ errors)))))"
+"          (list (string-append home-directory \"/.cron\")"
+"                (string-append (or (getenv \"XDG_CONFIG_HOME\")"
+"                                   (string-append home-directory \"/.config\"))"
+"                               \"/cron\")))"
+"    (if (eq? 2 errors)"
+"        (mcron-error 13"
+"                     \"Cannot read files in your ~/.config/cron (or ~/.cron) \""
+"                     \"directory.\"))))"
 "(define (valid-user user-name)"
 "  (setpwent)"
 "  (do ((entry (getpw) (getpw)))"
@@ -402,7 +418,7 @@ inner_main ()
 "   (if (not (option-ref options 'noetc #f))"
 "       (begin"
 "         (display"
-"          \"WARNING: cron will check for updates to /etc/crontab EVERY MINUTE. If you do\\n"
+"\"WARNING: cron will check for updates to /etc/crontab EVERY MINUTE. If you do\\n"
 "not use this file, or you are prepared to manually restart cron whenever you\\n"
 "make a change, then it is HIGHLY RECOMMENDED that you use the --noetc\\n"
 "option.\\n\")"
@@ -457,7 +473,7 @@ inner_main ()
 "        (run-job-loop fd-list)"
 "        (if (not (null? fd-list))"
 "            (process-update-request))))"
-                                                          ) );
+                     );
 }
 
 
